"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .httpclient import AsyncHttpClient, ClientOwner, HttpClient, close_clients
from .sdkconfiguration import SDKConfiguration
from .utils.logger import Logger, get_default_logger
from .utils.retries import RetryConfig
from fermisdk import models, utils
from fermisdk._hooks import SDKHooks
from fermisdk.types import OptionalNullable, UNSET
import httpx
import importlib
import sys
from typing import Any, Callable, Dict, Optional, TYPE_CHECKING, Union, cast
import weakref

if TYPE_CHECKING:
    from fermisdk.brain import Brain
    from fermisdk.chat_and_sessions import ChatAndSessions
    from fermisdk.data_connectors import DataConnectors


class Fermisdk(BaseSDK):
    r"""Fermi API: API documentation for Fermi platform including data connectors, brain visualization, and chat/session management"""

    brain: "Brain"
    r"""Knowledge graph visualization endpoints"""
    chat_and_sessions: "ChatAndSessions"
    r"""Chat and session management endpoints"""
    data_connectors: "DataConnectors"
    r"""Data connector endpoints for various integrations"""
    _sub_sdk_map = {
        "brain": ("fermisdk.brain", "Brain"),
        "chat_and_sessions": ("fermisdk.chat_and_sessions", "ChatAndSessions"),
        "data_connectors": ("fermisdk.data_connectors", "DataConnectors"),
    }

    def __init__(
        self,
        bearer_auth: Optional[Union[Optional[str], Callable[[], Optional[str]]]] = None,
        server_idx: Optional[int] = None,
        server_url: Optional[str] = None,
        url_params: Optional[Dict[str, str]] = None,
        client: Optional[HttpClient] = None,
        async_client: Optional[AsyncHttpClient] = None,
        retry_config: OptionalNullable[RetryConfig] = UNSET,
        timeout_ms: Optional[int] = None,
        debug_logger: Optional[Logger] = None,
    ) -> None:
        r"""Instantiates the SDK configuring it with the provided parameters.

        :param bearer_auth: The bearer_auth required for authentication
        :param server_idx: The index of the server to use for all methods
        :param server_url: The server URL to use for all methods
        :param url_params: Parameters to optionally template the server URL with
        :param client: The HTTP client to use for all synchronous methods
        :param async_client: The Async HTTP client to use for all asynchronous methods
        :param retry_config: The retry configuration to use for all supported methods
        :param timeout_ms: Optional request timeout applied to each operation in milliseconds
        """
        client_supplied = True
        if client is None:
            client = httpx.Client(follow_redirects=True)
            client_supplied = False

        assert issubclass(
            type(client), HttpClient
        ), "The provided client must implement the HttpClient protocol."

        async_client_supplied = True
        if async_client is None:
            async_client = httpx.AsyncClient(follow_redirects=True)
            async_client_supplied = False

        if debug_logger is None:
            debug_logger = get_default_logger()

        assert issubclass(
            type(async_client), AsyncHttpClient
        ), "The provided async_client must implement the AsyncHttpClient protocol."

        security: Any = None
        if callable(bearer_auth):
            # pylint: disable=unnecessary-lambda-assignment
            security = lambda: models.Security(bearer_auth=bearer_auth())
        else:
            security = models.Security(bearer_auth=bearer_auth)

        if server_url is not None:
            if url_params is not None:
                server_url = utils.template_url(server_url, url_params)

        BaseSDK.__init__(
            self,
            SDKConfiguration(
                client=client,
                client_supplied=client_supplied,
                async_client=async_client,
                async_client_supplied=async_client_supplied,
                security=security,
                server_url=server_url,
                server_idx=server_idx,
                retry_config=retry_config,
                timeout_ms=timeout_ms,
                debug_logger=debug_logger,
            ),
            parent_ref=self,
        )

        hooks = SDKHooks()

        # pylint: disable=protected-access
        self.sdk_configuration.__dict__["_hooks"] = hooks

        self.sdk_configuration = hooks.sdk_init(self.sdk_configuration)

        weakref.finalize(
            self,
            close_clients,
            cast(ClientOwner, self.sdk_configuration),
            self.sdk_configuration.client,
            self.sdk_configuration.client_supplied,
            self.sdk_configuration.async_client,
            self.sdk_configuration.async_client_supplied,
        )

    def dynamic_import(self, modname, retries=3):
        for attempt in range(retries):
            try:
                return importlib.import_module(modname)
            except KeyError:
                # Clear any half-initialized module and retry
                sys.modules.pop(modname, None)
                if attempt == retries - 1:
                    break
        raise KeyError(f"Failed to import module '{modname}' after {retries} attempts")

    def __getattr__(self, name: str):
        if name in self._sub_sdk_map:
            module_path, class_name = self._sub_sdk_map[name]
            try:
                module = self.dynamic_import(module_path)
                klass = getattr(module, class_name)
                instance = klass(self.sdk_configuration, parent_ref=self)
                setattr(self, name, instance)
                return instance
            except ImportError as e:
                raise AttributeError(
                    f"Failed to import module {module_path} for attribute {name}: {e}"
                ) from e
            except AttributeError as e:
                raise AttributeError(
                    f"Failed to find class {class_name} in module {module_path} for attribute {name}: {e}"
                ) from e

        raise AttributeError(
            f"'{type(self).__name__}' object has no attribute '{name}'"
        )

    def __dir__(self):
        default_attrs = list(super().__dir__())
        lazy_attrs = list(self._sub_sdk_map.keys())
        return sorted(list(set(default_attrs + lazy_attrs)))

    def __enter__(self):
        return self

    async def __aenter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if (
            self.sdk_configuration.client is not None
            and not self.sdk_configuration.client_supplied
        ):
            self.sdk_configuration.client.close()
        self.sdk_configuration.client = None

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if (
            self.sdk_configuration.async_client is not None
            and not self.sdk_configuration.async_client_supplied
        ):
            await self.sdk_configuration.async_client.aclose()
        self.sdk_configuration.async_client = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from typing import TYPE_CHECKING
from importlib import import_module
import builtins
import sys

if TYPE_CHECKING:
    from .connection import (
        Connection,
        ConnectionMetadata,
        ConnectionMetadataTypedDict,
        ConnectionTypedDict,
    )
    from .connectionlist import ConnectionList, ConnectionListTypedDict
    from .connectionmetadatarequest import (
        ConnectionMetadataRequest,
        ConnectionMetadataRequestMetadata,
        ConnectionMetadataRequestMetadataTypedDict,
        ConnectionMetadataRequestTypedDict,
    )
    from .connectionmetadataresponse import (
        ConnectionMetadataResponse,
        ConnectionMetadataResponseMetadata,
        ConnectionMetadataResponseMetadataTypedDict,
        ConnectionMetadataResponseTypedDict,
    )
    from .connectsessionrequest import (
        ConnectSessionRequest,
        ConnectSessionRequestMetadata,
        ConnectSessionRequestMetadataTypedDict,
        ConnectSessionRequestTypedDict,
    )
    from .connectsessionresponse import (
        ConnectSessionResponse,
        ConnectSessionResponseTypedDict,
    )
    from .databasemetadatarequest import (
        DatabaseMetadataRequest,
        DatabaseMetadataRequestDatabaseType,
        DatabaseMetadataRequestTypedDict,
    )
    from .databasemetadataresponse import (
        Database,
        DatabaseMetadataResponse,
        DatabaseMetadataResponseMetadata,
        DatabaseMetadataResponseMetadataTypedDict,
        DatabaseMetadataResponseTypedDict,
        DatabaseTypedDict,
    )
    from .deleteconnectionop import (
        DeleteConnectionRequest,
        DeleteConnectionRequestTypedDict,
        DeleteConnectionResponse,
        DeleteConnectionResponseTypedDict,
    )
    from .documentuploadresponse import (
        Document,
        DocumentTypedDict,
        DocumentUploadResponse,
        DocumentUploadResponseTypedDict,
    )
    from .getconnectionmetadataop import (
        GetConnectionMetadataRequest,
        GetConnectionMetadataRequestTypedDict,
    )
    from .getconnectionop import GetConnectionRequest, GetConnectionRequestTypedDict
    from .getsessioninfoop import GetSessionInfoRequest, GetSessionInfoRequestTypedDict
    from .listconnectionsop import (
        ListConnectionsRequest,
        ListConnectionsRequestTypedDict,
        ProviderConfigKey,
    )
    from .listsessionsop import ListSessionsRequest, ListSessionsRequestTypedDict
    from .performvisualizationop import (
        PerformVisualizationResponse,
        PerformVisualizationResponseTypedDict,
    )
    from .security import Security, SecurityTypedDict
    from .sessionactionrequest import (
        SessionActionRequest,
        SessionActionRequestAction,
        SessionActionRequestTypedDict,
    )
    from .sessionactionresponse import (
        SessionActionResponse,
        SessionActionResponseTypedDict,
    )
    from .sessioninfo import SessionInfo, SessionInfoTypedDict
    from .sessionlistresponse import SessionListResponse, SessionListResponseTypedDict
    from .sessionsummary import SessionSummary, SessionSummaryTypedDict
    from .syncenqueuerequest import SyncEnqueueRequest, SyncEnqueueRequestTypedDict
    from .syncenqueueresponse import SyncEnqueueResponse, SyncEnqueueResponseTypedDict
    from .testconnectionrequest import (
        TestConnectionRequest,
        TestConnectionRequestConnection,
        TestConnectionRequestConnectionTypedDict,
        TestConnectionRequestDatabaseType,
        TestConnectionRequestTypedDict,
    )
    from .testconnectionresponse import (
        TestConnectionResponse,
        TestConnectionResponseTypedDict,
    )
    from .uploaddocumentsop import (
        File,
        FileTypedDict,
        UploadDocumentsRequest,
        UploadDocumentsRequestTypedDict,
    )
    from .visualizationrequest import (
        ConfidenceThreshold,
        ResponseFormat,
        VisualizationRequest,
        VisualizationRequestAction,
        VisualizationRequestTypedDict,
    )
    from .visualizationresponsejson import (
        NodeProperties,
        NodePropertiesTypedDict,
        RelationshipDirection,
        RelationshipDirectionTypedDict,
        Statistics,
        StatisticsTypedDict,
        VisualizationResponseJSON,
        VisualizationResponseJSONMetadata,
        VisualizationResponseJSONMetadataTypedDict,
        VisualizationResponseJSONTypedDict,
    )

__all__ = [
    "ConfidenceThreshold",
    "ConnectSessionRequest",
    "ConnectSessionRequestMetadata",
    "ConnectSessionRequestMetadataTypedDict",
    "ConnectSessionRequestTypedDict",
    "ConnectSessionResponse",
    "ConnectSessionResponseTypedDict",
    "Connection",
    "ConnectionList",
    "ConnectionListTypedDict",
    "ConnectionMetadata",
    "ConnectionMetadataRequest",
    "ConnectionMetadataRequestMetadata",
    "ConnectionMetadataRequestMetadataTypedDict",
    "ConnectionMetadataRequestTypedDict",
    "ConnectionMetadataResponse",
    "ConnectionMetadataResponseMetadata",
    "ConnectionMetadataResponseMetadataTypedDict",
    "ConnectionMetadataResponseTypedDict",
    "ConnectionMetadataTypedDict",
    "ConnectionTypedDict",
    "Database",
    "DatabaseMetadataRequest",
    "DatabaseMetadataRequestDatabaseType",
    "DatabaseMetadataRequestTypedDict",
    "DatabaseMetadataResponse",
    "DatabaseMetadataResponseMetadata",
    "DatabaseMetadataResponseMetadataTypedDict",
    "DatabaseMetadataResponseTypedDict",
    "DatabaseTypedDict",
    "DeleteConnectionRequest",
    "DeleteConnectionRequestTypedDict",
    "DeleteConnectionResponse",
    "DeleteConnectionResponseTypedDict",
    "Document",
    "DocumentTypedDict",
    "DocumentUploadResponse",
    "DocumentUploadResponseTypedDict",
    "File",
    "FileTypedDict",
    "GetConnectionMetadataRequest",
    "GetConnectionMetadataRequestTypedDict",
    "GetConnectionRequest",
    "GetConnectionRequestTypedDict",
    "GetSessionInfoRequest",
    "GetSessionInfoRequestTypedDict",
    "ListConnectionsRequest",
    "ListConnectionsRequestTypedDict",
    "ListSessionsRequest",
    "ListSessionsRequestTypedDict",
    "NodeProperties",
    "NodePropertiesTypedDict",
    "PerformVisualizationResponse",
    "PerformVisualizationResponseTypedDict",
    "ProviderConfigKey",
    "RelationshipDirection",
    "RelationshipDirectionTypedDict",
    "ResponseFormat",
    "Security",
    "SecurityTypedDict",
    "SessionActionRequest",
    "SessionActionRequestAction",
    "SessionActionRequestTypedDict",
    "SessionActionResponse",
    "SessionActionResponseTypedDict",
    "SessionInfo",
    "SessionInfoTypedDict",
    "SessionListResponse",
    "SessionListResponseTypedDict",
    "SessionSummary",
    "SessionSummaryTypedDict",
    "Statistics",
    "StatisticsTypedDict",
    "SyncEnqueueRequest",
    "SyncEnqueueRequestTypedDict",
    "SyncEnqueueResponse",
    "SyncEnqueueResponseTypedDict",
    "TestConnectionRequest",
    "TestConnectionRequestConnection",
    "TestConnectionRequestConnectionTypedDict",
    "TestConnectionRequestDatabaseType",
    "TestConnectionRequestTypedDict",
    "TestConnectionResponse",
    "TestConnectionResponseTypedDict",
    "UploadDocumentsRequest",
    "UploadDocumentsRequestTypedDict",
    "VisualizationRequest",
    "VisualizationRequestAction",
    "VisualizationRequestTypedDict",
    "VisualizationResponseJSON",
    "VisualizationResponseJSONMetadata",
    "VisualizationResponseJSONMetadataTypedDict",
    "VisualizationResponseJSONTypedDict",
]

_dynamic_imports: dict[str, str] = {
    "Connection": ".connection",
    "ConnectionMetadata": ".connection",
    "ConnectionMetadataTypedDict": ".connection",
    "ConnectionTypedDict": ".connection",
    "ConnectionList": ".connectionlist",
    "ConnectionListTypedDict": ".connectionlist",
    "ConnectionMetadataRequest": ".connectionmetadatarequest",
    "ConnectionMetadataRequestMetadata": ".connectionmetadatarequest",
    "ConnectionMetadataRequestMetadataTypedDict": ".connectionmetadatarequest",
    "ConnectionMetadataRequestTypedDict": ".connectionmetadatarequest",
    "ConnectionMetadataResponse": ".connectionmetadataresponse",
    "ConnectionMetadataResponseMetadata": ".connectionmetadataresponse",
    "ConnectionMetadataResponseMetadataTypedDict": ".connectionmetadataresponse",
    "ConnectionMetadataResponseTypedDict": ".connectionmetadataresponse",
    "ConnectSessionRequest": ".connectsessionrequest",
    "ConnectSessionRequestMetadata": ".connectsessionrequest",
    "ConnectSessionRequestMetadataTypedDict": ".connectsessionrequest",
    "ConnectSessionRequestTypedDict": ".connectsessionrequest",
    "ConnectSessionResponse": ".connectsessionresponse",
    "ConnectSessionResponseTypedDict": ".connectsessionresponse",
    "DatabaseMetadataRequest": ".databasemetadatarequest",
    "DatabaseMetadataRequestDatabaseType": ".databasemetadatarequest",
    "DatabaseMetadataRequestTypedDict": ".databasemetadatarequest",
    "Database": ".databasemetadataresponse",
    "DatabaseMetadataResponse": ".databasemetadataresponse",
    "DatabaseMetadataResponseMetadata": ".databasemetadataresponse",
    "DatabaseMetadataResponseMetadataTypedDict": ".databasemetadataresponse",
    "DatabaseMetadataResponseTypedDict": ".databasemetadataresponse",
    "DatabaseTypedDict": ".databasemetadataresponse",
    "DeleteConnectionRequest": ".deleteconnectionop",
    "DeleteConnectionRequestTypedDict": ".deleteconnectionop",
    "DeleteConnectionResponse": ".deleteconnectionop",
    "DeleteConnectionResponseTypedDict": ".deleteconnectionop",
    "Document": ".documentuploadresponse",
    "DocumentTypedDict": ".documentuploadresponse",
    "DocumentUploadResponse": ".documentuploadresponse",
    "DocumentUploadResponseTypedDict": ".documentuploadresponse",
    "GetConnectionMetadataRequest": ".getconnectionmetadataop",
    "GetConnectionMetadataRequestTypedDict": ".getconnectionmetadataop",
    "GetConnectionRequest": ".getconnectionop",
    "GetConnectionRequestTypedDict": ".getconnectionop",
    "GetSessionInfoRequest": ".getsessioninfoop",
    "GetSessionInfoRequestTypedDict": ".getsessioninfoop",
    "ListConnectionsRequest": ".listconnectionsop",
    "ListConnectionsRequestTypedDict": ".listconnectionsop",
    "ProviderConfigKey": ".listconnectionsop",
    "ListSessionsRequest": ".listsessionsop",
    "ListSessionsRequestTypedDict": ".listsessionsop",
    "PerformVisualizationResponse": ".performvisualizationop",
    "PerformVisualizationResponseTypedDict": ".performvisualizationop",
    "Security": ".security",
    "SecurityTypedDict": ".security",
    "SessionActionRequest": ".sessionactionrequest",
    "SessionActionRequestAction": ".sessionactionrequest",
    "SessionActionRequestTypedDict": ".sessionactionrequest",
    "SessionActionResponse": ".sessionactionresponse",
    "SessionActionResponseTypedDict": ".sessionactionresponse",
    "SessionInfo": ".sessioninfo",
    "SessionInfoTypedDict": ".sessioninfo",
    "SessionListResponse": ".sessionlistresponse",
    "SessionListResponseTypedDict": ".sessionlistresponse",
    "SessionSummary": ".sessionsummary",
    "SessionSummaryTypedDict": ".sessionsummary",
    "SyncEnqueueRequest": ".syncenqueuerequest",
    "SyncEnqueueRequestTypedDict": ".syncenqueuerequest",
    "SyncEnqueueResponse": ".syncenqueueresponse",
    "SyncEnqueueResponseTypedDict": ".syncenqueueresponse",
    "TestConnectionRequest": ".testconnectionrequest",
    "TestConnectionRequestConnection": ".testconnectionrequest",
    "TestConnectionRequestConnectionTypedDict": ".testconnectionrequest",
    "TestConnectionRequestDatabaseType": ".testconnectionrequest",
    "TestConnectionRequestTypedDict": ".testconnectionrequest",
    "TestConnectionResponse": ".testconnectionresponse",
    "TestConnectionResponseTypedDict": ".testconnectionresponse",
    "File": ".uploaddocumentsop",
    "FileTypedDict": ".uploaddocumentsop",
    "UploadDocumentsRequest": ".uploaddocumentsop",
    "UploadDocumentsRequestTypedDict": ".uploaddocumentsop",
    "ConfidenceThreshold": ".visualizationrequest",
    "ResponseFormat": ".visualizationrequest",
    "VisualizationRequest": ".visualizationrequest",
    "VisualizationRequestAction": ".visualizationrequest",
    "VisualizationRequestTypedDict": ".visualizationrequest",
    "NodeProperties": ".visualizationresponsejson",
    "NodePropertiesTypedDict": ".visualizationresponsejson",
    "RelationshipDirection": ".visualizationresponsejson",
    "RelationshipDirectionTypedDict": ".visualizationresponsejson",
    "Statistics": ".visualizationresponsejson",
    "StatisticsTypedDict": ".visualizationresponsejson",
    "VisualizationResponseJSON": ".visualizationresponsejson",
    "VisualizationResponseJSONMetadata": ".visualizationresponsejson",
    "VisualizationResponseJSONMetadataTypedDict": ".visualizationresponsejson",
    "VisualizationResponseJSONTypedDict": ".visualizationresponsejson",
}


def dynamic_import(modname, retries=3):
    for attempt in range(retries):
        try:
            return import_module(modname, __package__)
        except KeyError:
            # Clear any half-initialized module and retry
            sys.modules.pop(modname, None)
            if attempt == retries - 1:
                break
    raise KeyError(f"Failed to import module '{modname}' after {retries} attempts")


def __getattr__(attr_name: str) -> object:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(
            f"No {attr_name} found in _dynamic_imports for module name -> {__name__} "
        )

    try:
        module = dynamic_import(module_name)
        result = getattr(module, attr_name)
        return result
    except ImportError as e:
        raise ImportError(
            f"Failed to import {attr_name} from {module_name}: {e}"
        ) from e
    except AttributeError as e:
        raise AttributeError(
            f"Failed to get {attr_name} from {module_name}: {e}"
        ) from e


def __dir__():
    lazy_attrs = builtins.list(_dynamic_imports.keys())
    return builtins.sorted(lazy_attrs)
